@@ -36,6 +36,7 @@
 using std::cout;
 using std::endl;
 using sbuild::_;
+using sbuild::auth;
 using boost::format;
 using namespace dchroot;
 
@@ -57,11 +58,11 @@ session::get_chroot_auth_status (sbuild::auth::status status,
 				 sbuild::chroot::ptr const& chroot) const
 {
   if (get_compat() == true)
-    status = change_auth(status, auth::STATUS_NONE);
+    status = auth::change_auth(status, auth::STATUS_NONE);
   else
-    status = change_auth(status,
-			 sbuild::session::get_chroot_auth_status(status,
-								 chroot));
+    status = auth::change_auth(status,
+			       sbuild::session::get_chroot_auth_status(status,
+								       chroot));
 
   return status;
 }
@@ -71,7 +72,7 @@ session::get_login_directories () const
 {
   sbuild::string_list ret;
 
-  std::string const& wd(get_wd());
+  std::string const& wd(get_auth()->get_wd());
   if (!wd.empty())
     {
       // Set specified working directory.
@@ -81,10 +82,10 @@ session::get_login_directories () const
     {
       // Set current working directory only if preserving environment.
       // Only change to home if not preserving the environment.
-      if (!get_environment().empty())
+      if (!get_auth()->get_environment().empty())
 	ret.push_back(this->sbuild::session::cwd);
       else
-	ret.push_back(get_home());
+	ret.push_back(get_auth()->get_home());
 
       // Final fallback to root.
       if (std::find(ret.begin(), ret.end(), "/") == ret.end())
@@ -113,11 +114,15 @@ session::get_user_command (sbuild::chroot::ptr& session_chroot,
   std::string commandstring = sbuild::string_list_to_string(command, " ");
   sbuild::log_debug(sbuild::DEBUG_NOTICE)
     << format("Running command: %1%") % commandstring << endl;
-  if (get_uid() == 0 || get_ruid() != get_uid())
+  if (get_auth()->get_uid() == 0 ||
+      get_auth()->get_ruid() != get_auth()->get_uid())
     syslog(LOG_USER|LOG_NOTICE, "[%s chroot] (%s->%s) Running command: \"%s\"",
-	   session_chroot->get_name().c_str(), get_ruser().c_str(), get_user().c_str(), commandstring.c_str());
+	   session_chroot->get_name().c_str(),
+	   get_auth()->get_ruser().c_str(),
+	   get_auth()->get_user().c_str(),
+	   commandstring.c_str());
 
-  if (get_verbosity() != auth::VERBOSITY_QUIET)
+  if (get_auth()->get_verbosity() != auth::VERBOSITY_QUIET)
     {
       std::string format_string;
       // TRANSLATORS: %1% = chroot name
