@@ -19,8 +19,9 @@
 #ifndef SBUILD_CHROOT_BLOCK_DEVICE_H
 #define SBUILD_CHROOT_BLOCK_DEVICE_H
 
-#include <sbuild/sbuild-chroot.h>
-#include <sbuild/sbuild-chroot-mountable.h>
+#include <sbuild/sbuild-config.h>
+#include <sbuild/sbuild-chroot-block-device-base.h>
+#include <sbuild/sbuild-chroot-lvm-snapshot.h>
 
 namespace sbuild
 {
@@ -30,14 +31,28 @@ namespace sbuild
    *
    * The device will be mounted on demand.
    */
-  class chroot_block_device : virtual public chroot,
-			      public chroot_mountable
+  class chroot_block_device : public chroot_block_device_base
   {
+  public:
+    /// Exception type.
+    typedef chroot::error error;
+
   protected:
     /// The constructor.
     chroot_block_device ();
 
+    /// The copy constructor.
+    chroot_block_device (const chroot_block_device& rhs);
+
+#ifdef SBUILD_FEATURE_LVMSNAP
+    /// The copy constructor.
+    chroot_block_device (const chroot_lvm_snapshot& rhs);
+#endif
+
     friend class chroot;
+#ifdef SBUILD_FEATURE_LVMSNAP
+    friend class chroot_lvm_snapshot;
+#endif
 
   public:
     /// The destructor.
@@ -46,36 +61,20 @@ namespace sbuild
     virtual chroot::ptr
     clone () const;
 
-    /**
-     * Get the block device of the chroot.
-     *
-     * @returns the device.
-     */
-    std::string const&
-    get_device () const;
+    virtual chroot::ptr
+    clone_session (std::string const& session_id,
+		   std::string const& user,
+		   bool               root) const;
 
-    /**
-     * Set the block device of the chroot.  This is the "source" device.
-     * It may be the case that the real device is different (for
-     * example, an LVM snapshot PV), but by default will be the device
-     * to mount.
-     *
-     * @param device the device.
-     */
-    void
-    set_device (std::string const& device);
-
-    virtual std::string const&
-    get_mount_device () const;
-
-    std::string const&
-    get_chroot_type () const;
+    virtual chroot::ptr
+    clone_source () const;
 
     virtual void
-    setup_env (environment& env);
+    setup_env (chroot const& chroot,
+	       environment& env) const;
 
     virtual session_flags
-    get_session_flags () const;
+    get_session_flags (chroot const& chroot) const;
 
   protected:
     virtual void
@@ -83,19 +82,19 @@ namespace sbuild
 		bool               lock,
 		int                status);
 
-    virtual void
-    get_details (format_detail& detail) const;
 
     virtual void
-    get_keyfile (keyfile& keyfile) const;
+    get_details (chroot const&  chroot,
+		 format_detail& detail) const;
 
     virtual void
-    set_keyfile (keyfile const& keyfile,
+    get_keyfile (chroot const& chroot,
+		 keyfile&      keyfile) const;
+
+    virtual void
+    set_keyfile (chroot&        chroot,
+		 keyfile const& keyfile,
 		 string_list&   used_keys);
-
-  private:
-    /// The block device to use.
-    std::string device;
   };
 
 }
