@@ -1,4 +1,4 @@
-/* Copyright © 2005-2008  Roger Leigh <rleigh@debian.org>
+/* Copyright © 2005-2009  Roger Leigh <rleigh@debian.org>
  *
  * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,11 +19,15 @@
 #include <config.h>
 
 #include "sbuild-chroot-lvm-snapshot.h"
+#include "sbuild-chroot-block-device.h"
+#include "sbuild-chroot-facet-session-clonable.h"
+#include "sbuild-chroot-facet-source-clonable.h"
+#include "sbuild-chroot-facet-mountable.h"
 #include "sbuild-format-detail.h"
 #include "sbuild-lock.h"
 
+#include <cassert>
 #include <cerrno>
-#include <iostream>
 
 #include <boost/format.hpp>
 
@@ -32,13 +36,18 @@ using boost::format;
 using namespace sbuild;
 
 chroot_lvm_snapshot::chroot_lvm_snapshot ():
-  chroot_block_device(),
-  chroot_source(),
+  chroot_block_device_base(),
   snapshot_device(),
   snapshot_options()
 {
-  set_run_setup_scripts(true);
-  set_run_exec_scripts(true);
+  add_facet(sbuild::chroot_facet_source_clonable::create());
+}
+
+chroot_lvm_snapshot::chroot_lvm_snapshot (const chroot_lvm_snapshot& rhs):
+  chroot_block_device_base(rhs),
+  snapshot_device(rhs.snapshot_device),
+  snapshot_options(rhs.snapshot_options)
+{
 }
 
 chroot_lvm_snapshot::~chroot_lvm_snapshot ()
@@ -51,12 +60,31 @@ chroot_lvm_snapshot::clone () const
   return ptr(new chroot_lvm_snapshot(*this));
 }
 
+sbuild::chroot::ptr
+chroot_lvm_snapshot::clone_session (std::string const& session_id,
+				    std::string const& user,
+				    bool               root) const
+{
+  chroot_facet_session_clonable::const_ptr psess
+    (get_facet<chroot_facet_session_clonable>());
+  assert(psess);
+
+  ptr session(new chroot_lvm_snapshot(*this));
+  psess->clone_session_setup(session, session_id, user, root);
+
+  return session;
+}
+
 sbuild::chroot::ptr
 chroot_lvm_snapshot::clone_source () const
 {
   ptr clone(new chroot_block_device(*this));
 
-  chroot_source::clone_source_setup(clone);
+  chroot_facet_source_clonable::const_ptr psrc
+    (get_facet<chroot_facet_source_clonable>());
+  assert(psrc);
+
+  psrc->clone_source_setup(clone);
 
   return clone;
 }
@@ -74,12 +102,10 @@ chroot_lvm_snapshot::set_snapshot_device (std::string const& snapshot_device)
     throw error(snapshot_device, DEVICE_ABS);
 
   this->snapshot_device = snapshot_device;
-}
 
-std::string const&
-chroot_lvm_snapshot::get_mount_device () const
-{
-  return this->snapshot_device;
+  chroot_facet_mountable::ptr pmnt
+    (get_facet<chroot_facet_mountable>());
+  pmnt->set_mount_device(this->snapshot_device);
 }
 
 std::string const&
@@ -103,10 +129,10 @@ chroot_lvm_snapshot::get_chroot_type () const
 }
 
 void
-chroot_lvm_snapshot::setup_env (environment& env)
+chroot_lvm_snapshot::setup_env (chroot const& chroot,
+				environment&  env) const
 {
-  chroot_block_device::setup_env(env);
-  chroot_source::setup_env(env);
+  chroot_block_device_base::setup_env(chroot, env);
 
   env.add("CHROOT_LVM_SNAPSHOT_NAME", sbuild::basename(get_snapshot_device()));
   env.add("CHROOT_LVM_SNAPSHOT_DEVICE", get_snapshot_device());
@@ -141,11 +167,6 @@ chroot_lvm_snapshot::setup_lock (chroot::setup_type type,
 	    }
 	  else
 	    {
-	      /* Lock is preserved while running a command. */
-	      if ((type == EXEC_START && lock == false) ||
-		  (type == EXEC_STOP && lock == true))
-		return;
-
 	      sbuild::device_lock dlock(device);
 	      if (lock)
 		{
@@ -192,16 +213,21 @@ chroot_lvm_snapshot::setup_lock (chroot::setup_type type,
 }
 
 sbuild::chroot::session_flags
-chroot_lvm_snapshot::get_session_flags () const
+chroot_lvm_snapshot::get_session_flags (chroot const& chroot) const
 {
-  return SESSION_CREATE | chroot_source::get_session_flags();
+  session_flags flags = SESSION_NOFLAGS;
+
+  if (get_active())
+    flags = flags | SESSION_PURGE;
+
+  return flags;
 }
 
 void
-chroot_lvm_snapshot::get_details (format_detail& detail) const
+chroot_lvm_snapshot::get_details (chroot const& chroot,
+				  format_detail& detail) const
 {
-  chroot_block_device::get_details(detail);
-  chroot_source::get_details(detail);
+  chroot_block_device_base::get_details(chroot, detail);
 
   if (!this->snapshot_device.empty())
     detail.add(_("LVM Snapshot Device"), get_snapshot_device());
@@ -210,35 +236,42 @@ chroot_lvm_snapshot::get_details (format_detail& detail) const
 }
 
 void
-chroot_lvm_snapshot::get_keyfile (keyfile& keyfile) const
+chroot_lvm_snapshot::get_keyfile (chroot const& chroot,
+				  keyfile& keyfile) const
 {
-  chroot_block_device::get_keyfile(keyfile);
-  chroot_source::get_keyfile(keyfile);
+  chroot_block_device_base::get_keyfile(chroot, keyfile);
 
   if (get_active())
-    keyfile::set_object_value(*this, &chroot_lvm_snapshot::get_snapshot_device,
-			      keyfile, get_name(), "lvm-snapshot-device");
+    keyfile::set_object_value(*this,
+			      &chroot_lvm_snapshot::get_snapshot_device,
+			      keyfile, get_keyfile_name(),
+			      "lvm-snapshot-device");
 
-  keyfile::set_object_value(*this, &chroot_lvm_snapshot::get_snapshot_options,
-			    keyfile, get_name(), "lvm-snapshot-options");
+  if (!get_active())
+    keyfile::set_object_value(*this,
+			      &chroot_lvm_snapshot::get_snapshot_options,
+			      keyfile, get_keyfile_name(),
+			      "lvm-snapshot-options");
 }
 
 void
-chroot_lvm_snapshot::set_keyfile (keyfile const& keyfile,
+chroot_lvm_snapshot::set_keyfile (chroot&        chroot,
+				  keyfile const& keyfile,
 				  string_list&   used_keys)
 {
-  chroot_block_device::set_keyfile(keyfile, used_keys);
-  chroot_source::set_keyfile(keyfile, used_keys);
+  chroot_block_device_base::set_keyfile(chroot, keyfile, used_keys);
 
   keyfile::get_object_value(*this, &chroot_lvm_snapshot::set_snapshot_device,
-			    keyfile, get_name(), "lvm-snapshot-device",
+			    keyfile, get_keyfile_name(), "lvm-snapshot-device",
 			    get_active() ?
 			    keyfile::PRIORITY_REQUIRED :
 			    keyfile::PRIORITY_DISALLOWED);
   used_keys.push_back("lvm-snapshot-device");
 
   keyfile::get_object_value(*this, &chroot_lvm_snapshot::set_snapshot_options,
-			    keyfile, get_name(), "lvm-snapshot-options",
-			    keyfile::PRIORITY_REQUIRED);
+			    keyfile, get_keyfile_name(), "lvm-snapshot-options",
+			    get_active() ?
+			    keyfile::PRIORITY_DEPRECATED :
+			    keyfile::PRIORITY_REQUIRED); // Only needed for creating snapshot, not using snapshot
   used_keys.push_back("lvm-snapshot-options");
 }
