@@ -23,16 +23,18 @@
 #include <sbuild/sbuild-environment.h>
 #include <sbuild/sbuild-format-detail.h>
 #include <sbuild/sbuild-keyfile.h>
-#include <sbuild/sbuild-personality.h>
 #include <sbuild/sbuild-regex.h>
 #include <sbuild/sbuild-tr1types.h>
 
+#include <list>
 #include <ostream>
 #include <string>
 
 namespace sbuild
 {
 
+  class chroot_facet;
+
   /**
    * Common chroot data.  This class contains all of the metadata
    * associated with a single chroot, for all chroot types.  This is
@@ -48,18 +50,16 @@ namespace sbuild
       {
 	SETUP_START,   ///< Activate a chroot.
 	SETUP_RECOVER, ///< Reactivate a chroot.
-	SETUP_STOP,    ///< Deactivate a chroot.
-	EXEC_START,    ///< Start executing a command in an active chroot.
-	EXEC_STOP      ///< End executing a command in an active chroot.
+	SETUP_STOP     ///< Deactivate a chroot.
       };
 
     /// Chroot session properties
     enum session_flags
       {
-	SESSION_NOFLAGS = 0,     ///< No flags are set.
-	SESSION_CREATE = 1 << 0, ///< The chroot supports session creation.
-	SESSION_CLONE  = 1 << 1, ///< The chroot supports cloning.
-	SESSION_PURGE  = 1 << 2  ///< The chroot should be purged.
+	SESSION_NOFLAGS = 0,      ///< No flags are set.
+	SESSION_CREATE  = 1 << 0, ///< The chroot supports session creation.
+	SESSION_CLONE   = 1 << 1, ///< The chroot supports cloning.
+	SESSION_PURGE   = 1 << 2  ///< The chroot should be purged.
       };
 
     /// Error codes.
@@ -72,6 +72,9 @@ namespace sbuild
 	DEVICE_LOCK,     ///< Failed to lock device.
 	DEVICE_NOTBLOCK, ///< File is not a block device.
 	DEVICE_UNLOCK,   ///< Failed to unlock device.
+	DIRECTORY_ABS,   ///< Directory must have an absolute path.
+	FACET_INVALID,   ///< Attempt to add object which is not a facet.
+	FACET_PRESENT,   ///< Attempt to add facet which is already in use.
 	FILE_ABS,        ///< File must have an absolute path.
 	FILE_LOCK,       ///< Failed to acquire lock.
 	FILE_NOTREG,     ///< File is not a regular file.
@@ -89,10 +92,16 @@ namespace sbuild
     /// A shared_ptr to a chroot object.
     typedef std::tr1::shared_ptr<chroot> ptr;
 
+    /// A shared_ptr to a const chroot object.
+    typedef std::tr1::shared_ptr<const chroot> const_ptr;
+
   protected:
     /// The constructor.
     chroot ();
 
+    /// The copy constructor.
+    chroot (const chroot& rhs);
+
   public:
     /// The destructor.
     virtual ~chroot ();
@@ -114,6 +123,27 @@ namespace sbuild
     virtual ptr
     clone () const = 0;
 
+    /**
+     * Create a session chroot.
+     *
+     * @param session_id the identifier for the new session.
+     * @param user the user creating the session.
+     * @param root true if the user has root access, otherwise false.
+     * @returns a session chroot.
+     */
+    virtual chroot::ptr
+    clone_session (std::string const& session_id,
+		   std::string const& user,
+		   bool               root) const = 0;
+
+    /**
+     * Create a source chroot.
+     *
+     * @returns a source chroot.
+     */
+    virtual chroot::ptr
+    clone_source () const = 0;
+
     /**
      * Get the name of the chroot.
      *
@@ -130,6 +160,33 @@ namespace sbuild
     void
     set_name (std::string const& name);
 
+    /**
+     * Get the session ID of the chroot.
+     *
+     * @returns the session ID.
+     */
+    std::string const&
+    get_session_id () const;
+
+    /**
+     * Set the session ID of the chroot.
+     * @todo Create as chroot facet
+     *
+     * @param session_id the session ID.
+     */
+    void
+    set_session_id (std::string const& session_id);
+
+    /**
+     * Get the keyfile name of the chroot.
+     * Depending on whether the chroot is active or not, this will
+     * return the session ID or the chroot name, respectively.
+     *
+     * @returns the name.
+     */
+    std::string const&
+    get_keyfile_name () const;
+
     /**
      * Get the description of the chroot.
      *
@@ -151,7 +208,7 @@ namespace sbuild
      *
      * @returns the mount location.
      */
-    virtual std::string const&
+    std::string const&
     get_mount_location () const;
 
     /**
@@ -162,27 +219,6 @@ namespace sbuild
     void
     set_mount_location (std::string const& location);
 
-    /**
-     * Get the location of the chroot.  This is the path to the root
-     * of the chroot, and is typically the same as the mount location,
-     * but is overridden by the chroot type if required.
-     *
-     * @returns the mount location.
-     */
-    virtual std::string const&
-    get_location () const;
-
-  protected:
-    /**
-     * Set the location of the chroot.  This is the path to the root
-     * of the chroot, and is typically the same as the mount location,
-     * but is overridden by the chroot type if required.
-     *
-     * @param location the mount location.
-     */
-    virtual void
-    set_location (std::string const& location);
-
   public:
     /**
      * Get the path to the chroot.  This is the absolute path to the
@@ -193,23 +229,7 @@ namespace sbuild
      * @returns the path.
      */
     virtual std::string
-    get_path () const;
-
-    /**
-     * Get the mount device of the chroot.
-     *
-     * @returns the device.
-     */
-    virtual std::string const&
-    get_mount_device () const;
-
-    /**
-     * Set the mount device of the chroot.
-     *
-     * @param device the device.
-     */
-    void
-    set_mount_device (std::string const& device);
+    get_path () const = 0;
 
     /**
      * Get the priority of the chroot.  This is a number indicating
@@ -343,21 +363,14 @@ namespace sbuild
     set_environment_filter (regex const& environment_filter);
 
     /**
-     * Get the activity status of the chroot.
+     * Get the activity status of the chroot.  The chroot is active if
+     * it has been cloned as a session.
      *
      * @returns true if active, false if inactive
      */
     bool
     get_active () const;
 
-    /**
-     * Set the activity status of the chroot.
-     *
-     * @param active true if active, false if inactive
-     */
-    void
-    set_active (bool active);
-
     /**
      * Get the originality of the chroot.
      *
@@ -369,7 +382,7 @@ namespace sbuild
     /**
      * Set the originality of the chroot.
      *
-     * @param original true if original, false if geneated.
+     * @param original true if original, false if generated.
      */
     void
     set_original (bool original);
@@ -382,6 +395,7 @@ namespace sbuild
     bool
     get_run_setup_scripts () const;
 
+  protected:
     /**
      * Set whether chroot setup scripts will be run.
      *
@@ -391,23 +405,7 @@ namespace sbuild
     void
     set_run_setup_scripts (bool run_setup_scripts);
 
-    /**
-     * Check if chroot exec scripts will be run.
-     *
-     * @returns true if exec scripts will be run, otherwise false.
-     */
-    bool
-    get_run_exec_scripts () const;
-
-    /**
-     * Set whether chroot exec scripts will be run.
-     *
-     * @param run_exec_scripts true if exec scripts will be run,
-     * otherwise false.
-     */
-    void
-    set_run_exec_scripts (bool run_exec_scripts);
-
+  public:
     /**
      * Get the script configuration file for the chroot.  This is a
      * filename, either relative to the configured pkgsysconfdir or an
@@ -446,22 +444,6 @@ namespace sbuild
     void
     set_command_prefix (string_list const& command_prefix);
 
-    /**
-     * Get the process execution domain for the chroot.
-     *
-     * @returns the personality.
-     */
-    personality const&
-    get_persona () const;
-
-    /**
-     * Set the process execution domain for the chroot.
-     *
-     * @param persona the personality.
-     */
-    void
-    set_persona (personality const& persona);
-
     /**
      * Get the type of the chroot.
      *
@@ -476,8 +458,19 @@ namespace sbuild
      *
      * @param env the environment to set.
      */
+    void
+    setup_env (environment& env) const;
+
+    /**
+     * Set environment.  Set the environment that the setup scripts
+     * will see during execution.
+     *
+     * @param chroot the chroot to use.
+     * @param env the environment to set.
+     */
     virtual void
-    setup_env (environment& env);
+    setup_env (chroot const& chroot,
+	       environment& env) const = 0;
 
     /**
      * Lock a chroot during setup.  The locking technique (if any) may
@@ -539,14 +532,51 @@ namespace sbuild
 	       int        status) = 0;
 
   public:
+    template <typename T>
+    std::tr1::shared_ptr<T>
+    get_facet ();
+
+    template <typename T>
+    const std::tr1::shared_ptr<const T>
+    get_facet () const;
+
+    template <typename T>
+    void
+    add_facet (std::tr1::shared_ptr<T> facet);
+
+    template <typename T>
+    void
+    remove_facet ();
+
+    template <typename T>
+    void
+    remove_facet (std::tr1::shared_ptr<T> facet);
+
+    template <typename T>
+    void
+    replace_facet (std::tr1::shared_ptr<T> facet);
+
+    string_list
+    list_facets () const;
+
     /**
      * Get the session flags of the chroot.  These determine how the
      * Session controlling the chroot will operate.
      *
      * @returns the session flags.
      */
-    virtual session_flags
-    get_session_flags () const = 0;
+    session_flags
+    get_session_flags () const;
+
+    /**
+     * Get the session flags of the chroot.  These determine how the
+     * Session controlling the chroot will operate.
+     *
+     * @param chroot the chroot to use.
+     * @returns the session flags.
+     */
+    virtual chroot::session_flags
+    get_session_flags (chroot const& chroot) const = 0;
 
     /**
      * Print detailed information about the chroot to a stream.  The
@@ -591,16 +621,24 @@ namespace sbuild
       return keyfile;
     }
 
-
-  protected:
     /**
      * Get detailed information about the chroot for output.
      *
      * @param detail the details to output to.
      */
-    virtual void
+    void
     get_details (format_detail& detail) const;
 
+    /**
+     * Get detailed information about the chroot for output.
+     *
+     * @param chroot the chroot to use.
+     * @param detail the details to output to.
+     */
+    virtual void
+    get_details (chroot const&  chroot,
+		 format_detail& detail) const = 0;
+
     /**
      * Print detailed information about the chroot to a stream.  The
      * information is printed in plain text with one line per
@@ -618,9 +656,23 @@ namespace sbuild
      *
      * @param keyfile the keyfile to use.
      */
-    virtual void
+    void
     get_keyfile (keyfile& keyfile) const;
 
+  protected:
+    /**
+     * Copy the chroot properties into a keyfile.  The keyfile group
+     * with the name of the chroot will be set; if it already exists,
+     * it will be removed before setting it.
+     *
+     * @param chroot the chroot to use.
+     * @param keyfile the keyfile to use.
+     */
+    virtual void
+    get_keyfile (chroot const& chroot,
+		 keyfile&      keyfile) const = 0;
+
+  public:
     /**
      * Set the chroot properties from a keyfile.  The chroot name must
      * have previously been set, so that the correct keyfile group may
@@ -629,16 +681,33 @@ namespace sbuild
      * @param keyfile the keyfile to get the properties from.
      * @param used_keys a list of the keys used will be set.
      */
-    virtual void
+    void
     set_keyfile (keyfile const& keyfile,
 		 string_list&   used_keys);
 
+  protected:
+    /**
+     * Set the chroot properties from a keyfile.  The chroot name must
+     * have previously been set, so that the correct keyfile group may
+     * be determined.
+     *
+     * @param chroot the chroot to use.
+     * @param keyfile the keyfile to get the properties from.
+     * @param used_keys a list of the keys used will be set.
+     */
+    virtual void
+    set_keyfile (chroot&        chroot,
+		 keyfile const& keyfile,
+		 string_list&   used_keys) = 0;
+
   private:
     /// Chroot name.
     std::string   name;
+    /// Chroot session ID.
+    std::string   session_id;
     /// Chroot description.
     std::string   description;
-    /// Chroot prioroty.
+    /// Chroot priority.
     unsigned int  priority;
     /// Users allowed to access the chroot.
     string_list   users;
@@ -654,24 +723,18 @@ namespace sbuild
     regex         environment_filter;
     /// Location to mount chroot in the filesystem (if any).
     std::string   mount_location;
-    /// Location inside the mount location root.
-    std::string   location;
-    /// Block device to mount (if any).
-    std::string   mount_device;
-    /// Chroot activity status.
-    bool          active;
     /// Was the chroot automatically generated?
     bool          original;
     /// Run chroot setup scripts?
     bool          run_setup_scripts;
-    /// Run chroot exec scripts?
-    bool          run_exec_scripts;
     /// Configuration of the setup and exec scripts.
     std::string   script_config;
     /// Command prefix.
     string_list   command_prefix;
-    /// Process execution domain (Linux only).
-    personality   persona;
+
+    typedef std::tr1::shared_ptr<chroot_facet> facet_ptr;
+    typedef std::list<facet_ptr> facet_list;
+    facet_list facets;
   };
 
   /**
@@ -704,6 +767,98 @@ namespace sbuild
 
 }
 
+#include <sbuild/sbuild-chroot-facet.h>
+
+namespace sbuild
+{
+
+  template <typename T>
+  std::tr1::shared_ptr<T>
+  chroot::get_facet ()
+  {
+    std::tr1::shared_ptr<T> ret;
+
+    for (facet_list::const_iterator pos = facets.begin();
+	 pos != facets.end();
+	 ++pos)
+      {
+	if (ret = std::tr1::dynamic_pointer_cast<T>(*pos))
+	  break;
+      }
+
+    return ret;
+  }
+
+  template <typename T>
+  const std::tr1::shared_ptr<const T>
+  chroot::get_facet () const
+  {
+    std::tr1::shared_ptr<T> ret;
+
+    for (facet_list::const_iterator pos = facets.begin();
+	 pos != facets.end();
+	 ++pos)
+      {
+	if (ret = std::tr1::dynamic_pointer_cast<T>(*pos))
+	  break;
+      }
+
+    return std::tr1::const_pointer_cast<T>(ret);
+  }
+
+  template <typename T>
+  void
+  chroot::add_facet (std::tr1::shared_ptr<T> facet)
+  {
+    facet_ptr new_facet = std::tr1::dynamic_pointer_cast<chroot_facet>(facet);
+    if (!new_facet)
+      throw error(FACET_INVALID);
+
+    for (facet_list::const_iterator pos = facets.begin();
+	 pos != facets.end();
+	 ++pos)
+      {
+	if (std::tr1::dynamic_pointer_cast<T>(*pos))
+	  throw error(FACET_PRESENT);
+      }
+
+    new_facet->set_chroot(*this);
+    facets.push_back(new_facet);
+  }
+
+  template <typename T>
+  void
+  chroot::remove_facet ()
+  {
+    for (facet_list::iterator pos = facets.begin();
+	 pos != facets.end();
+	 ++pos)
+      {
+	if (std::tr1::dynamic_pointer_cast<T>(*pos))
+	  {
+	    facets.erase(pos);
+	    break;
+	  }
+      }
+  }
+
+  template <typename T>
+  void
+  chroot::remove_facet (std::tr1::shared_ptr<T> facet)
+  {
+    remove_facet<T>();
+  }
+
+  template <typename T>
+  void
+  chroot::replace_facet (std::tr1::shared_ptr<T> facet)
+  {
+    remove_facet<T>();
+    add_facet(facet);
+  }
+
+}
+
 #endif /* SBUILD_CHROOT_H */
 
 /*
