@@ -1,4 +1,4 @@
-/* Copyright © 2005-2007  Roger Leigh <rleigh@debian.org>
+/* Copyright © 2005-2009  Roger Leigh <rleigh@debian.org>
  *
  * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,8 +18,11 @@
 
 #include <config.h>
 
-#include "sbuild-chroot-plain.h"
-#include "sbuild-chroot-lvm-snapshot.h"
+#include "sbuild-chroot-config.h"
+#include "sbuild-chroot-facet-personality.h"
+#include "sbuild-chroot-facet-session.h"
+#include "sbuild-chroot-facet-session-clonable.h"
+#include "sbuild-auth-null.h"
 #include "sbuild-ctty.h"
 #include "sbuild-run-parts.h"
 #include "sbuild-session.h"
@@ -42,12 +45,6 @@
 
 #include <boost/format.hpp>
 
-#ifdef HAVE_UUID
-#include <uuid/uuid.h>
-#else
-#include <time.h>
-#endif
-
 using std::cout;
 using std::endl;
 using boost::format;
@@ -121,27 +118,6 @@ namespace
       emap(session::USER_SWITCH,    N_("(%1%->%2%): User switching is not permitted")),
     };
 
-  /**
-   * Get the current working directory.  If it can't be found, fall
-   * back to root.
-   *
-   * @returns the current working directory.
-   */
-  std::string
-  getcwd ()
-  {
-    std::string cwd;
-
-    char *raw_cwd = ::getcwd (0, 0);
-    if (raw_cwd)
-      cwd = raw_cwd;
-    else
-      cwd = "/";
-    free(raw_cwd);
-
-    return cwd;
-  }
-
   /**
    * Check group membership.
    *
@@ -149,27 +125,27 @@ namespace
    * @returns true if the user is a member of group, otherwise false.
    */
   bool
-  is_group_member (std::string const& group)
+  is_group_member (std::string const& groupname)
   {
     errno = 0;
-    struct group *groupbuf = getgrnam(group.c_str());
-    if (groupbuf == 0)
+    sbuild::group grp(groupname);
+    if (!grp)
       {
 	if (errno == 0)
 	  {
-	    session::error e(group, session::GROUP_UNKNOWN);
+	    session::error e(groupname, session::GROUP_UNKNOWN);
 	    log_exception_warning(e);
 	  }
 	else
 	  {
-	    session::error e(group, session::GROUP_UNKNOWN, strerror(errno));
+	    session::error e(groupname, session::GROUP_UNKNOWN, strerror(errno));
 	    log_exception_warning(e);
 	  }
 	return false;
       }
 
     bool group_member = false;
-    if (groupbuf->gr_gid == getgid())
+    if (grp.gr_gid == getgid())
       {
 	group_member = true;
       }
@@ -183,11 +159,15 @@ namespace
 	    gid_t *supp_groups = new gid_t[supp_group_count];
 	    assert (supp_groups);
 	    if (getgroups(supp_group_count, supp_groups) < 1)
-	      throw session::error(session::GROUP_GET_SUP, strerror(errno));
+	      {
+		// Free supp_groups before throwing to avoid leak.
+		delete[] supp_groups;
+		throw session::error(session::GROUP_GET_SUP, strerror(errno));
+	      }
 
 	    for (int i = 0; i < supp_group_count; ++i)
 	      {
-		if (groupbuf->gr_gid == supp_groups[i])
+		if (grp.gr_gid == supp_groups[i])
 		  group_member = true;
 	      }
 	    delete[] supp_groups;
@@ -240,12 +220,12 @@ session::session (std::string const&         service,
 		  config_ptr&                config,
 		  operation                  operation,
 		  sbuild::string_list const& chroots):
-  auth(service),
+  authstat(auth_null::create(service)),
   config(config),
   chroots(chroots),
   chroot_status(true),
   lock_status(true),
-  child_status(0),
+  child_status(EXIT_FAILURE),
   session_operation(operation),
   session_id(),
   force(false),
@@ -253,7 +233,7 @@ session::session (std::string const&         service,
   saved_sigterm_signal(),
   saved_termios(),
   termios_ok(false),
-  cwd(getcwd())
+  cwd(sbuild::getcwd())
 {
 }
 
@@ -261,6 +241,18 @@ session::~session ()
 {
 }
 
+auth::ptr const&
+session::get_auth () const
+{
+  return this->authstat;
+}
+
+void
+session::set_auth (auth::ptr& auth)
+{
+  this->authstat = auth;
+}
+
 session::config_ptr const&
 session::get_config () const
 {
@@ -324,7 +316,7 @@ session::set_force (bool force)
 void
 session::save_termios ()
 {
-  string_list const& command(auth::get_command());
+  string_list const& command(this->authstat->get_command());
 
   this->termios_ok = false;
 
@@ -346,7 +338,7 @@ session::save_termios ()
 void
 session::restore_termios ()
 {
-  string_list const& command(auth::get_command());
+  string_list const& command(this->authstat->get_command());
 
   // Restore if running a login shell and have a controlling terminal,
   // and have previously saved the terminal state.
@@ -367,27 +359,30 @@ session::get_child_status () const
   return this->child_status;
 }
 
-auth::status
-session::get_chroot_auth_status (auth::status status,
-				 chroot::ptr const& chroot) const
+void
+session::get_chroot_membership (chroot::ptr const& chroot,
+				bool&              in_users,
+				bool&              in_root_users,
+				bool&              in_groups,
+				bool&              in_root_groups) const
 {
   string_list const& users = chroot->get_users();
   string_list const& root_users = chroot->get_root_users();
   string_list const& groups = chroot->get_groups();
   string_list const& root_groups = chroot->get_root_groups();
 
-  bool in_users = false;
-  bool in_root_users = false;
-  bool in_groups = false;
-  bool in_root_groups = false;
+  in_users = false;
+  in_root_users = false;
+  in_groups = false;
+  in_root_groups = false;
 
   sbuild::string_list::const_iterator upos =
-    find(users.begin(), users.end(), get_ruser());
+    find(users.begin(), users.end(), this->authstat->get_ruser());
   if (upos != users.end())
     in_users = true;
 
   sbuild::string_list::const_iterator rupos =
-    find(root_users.begin(), root_users.end(), get_ruser());
+    find(root_users.begin(), root_users.end(), this->authstat->get_ruser());
   if (rupos != root_users.end())
     in_root_users = true;
 
@@ -415,6 +410,21 @@ session::get_chroot_auth_status (auth::status status,
     << "In root-users: " << in_root_users << endl
     << "In root-groups: " << in_root_groups << endl;
 
+}
+
+auth::status
+session::get_chroot_auth_status (auth::status status,
+				 chroot::ptr const& chroot) const
+{
+  bool in_users = false;
+  bool in_root_users = false;
+  bool in_groups = false;
+  bool in_root_groups = false;
+
+  get_chroot_membership(chroot,
+			in_users, in_root_users,
+			in_groups, in_root_groups);
+
   /*
    * No auth required if in root users or root groups and
    * changing to root, or if the uid is not changing.  If not
@@ -422,26 +432,26 @@ session::get_chroot_auth_status (auth::status status,
    */
   if ((in_users == true || in_groups == true ||
        in_root_users == true || in_root_groups == true) &&
-      this->get_ruid() == this->get_uid())
+      this->authstat->get_ruid() == this->authstat->get_uid())
     {
-      status = change_auth(status, auth::STATUS_NONE);
+      status = auth::change_auth(status, auth::STATUS_NONE);
     }
   else if ((in_root_users == true || in_root_groups == true) &&
-	   this->get_uid() == 0)
+	   this->authstat->get_uid() == 0)
     {
-      status = change_auth(status, auth::STATUS_NONE);
+      status = auth::change_auth(status, auth::STATUS_NONE);
     }
   else if (in_users == true || in_groups == true)
     // Auth required if not in root group
     {
-      status = change_auth(status, auth::STATUS_USER);
+      status = auth::change_auth(status, auth::STATUS_USER);
     }
   else // Not in any groups
     {
-      if (this->get_ruid() == 0)
-	status = change_auth(status, auth::STATUS_USER);
+      if (this->authstat->get_ruid() == 0)
+	status = auth::change_auth(status, auth::STATUS_USER);
       else
-	status = change_auth(status, auth::STATUS_FAIL);
+	status = auth::change_auth(status, auth::STATUS_FAIL);
     }
 
   return status;
@@ -475,15 +485,70 @@ session::get_auth_status () const
 	{
 	  error e(*cur, CHROOT_ALIAS);
 	  log_exception_warning(e);
-	  status = change_auth(status, auth::STATUS_FAIL);
+	  status = auth::change_auth(status, auth::STATUS_FAIL);
 	}
 
-      status = change_auth(status, get_chroot_auth_status(status, chroot));
+      status = auth::change_auth(status, get_chroot_auth_status(status, chroot));
     }
 
   return status;
 }
 
+void
+session::run ()
+{
+  try
+    {
+      this->authstat->start();
+      this->authstat->authenticate(get_auth_status());
+      this->authstat->setupenv();
+      this->authstat->account();
+      try
+	{
+	  this->authstat->cred_establish();
+
+	  run_impl();
+
+	  /* The session is now finished, either
+	     successfully or not.  All PAM operations are
+	     now for cleanup and shutdown, and we must
+	     clean up whether or not errors were raised at
+	     any previous point.  This means only the
+	     first error is reported back to the user. */
+
+	  /* Don't cope with failure, since we are now
+	     already bailing out, and an error may already
+	     have been raised */
+	}
+      catch (auth::error const& e)
+	{
+	  try
+	    {
+	      this->authstat->cred_delete();
+	    }
+	  catch (auth::error const& discard)
+	    {
+	    }
+	  throw;
+	}
+      this->authstat->cred_delete();
+    }
+  catch (auth::error const& e)
+    {
+      try
+	{
+	  /* Don't cope with failure, since we are now already bailing out,
+	     and an error may already have been raised */
+	  this->authstat->stop();
+	}
+      catch (auth::error const& discard)
+	{
+	}
+      throw;
+    }
+  this->authstat->stop();
+}
+
 void
 session::run_impl ()
 {
@@ -509,138 +574,90 @@ session::run_impl ()
 	  if (!ch) // Should never happen, but cater for it anyway.
 	    throw error(*cur, CHROOT_UNKNOWN);
 
+	  // For now, use a copy of the chroot; if we create a session
+	  // later, we will replace it.
 	  chroot::ptr chroot(ch->clone());
+	  assert(chroot);
 
-	  /* If restoring a session, set the session ID from the
-	     chroot name, or else generate it.  Only chroots which
-	     support session creation append a UUID to the session
-	     ID. */
-	  if (chroot->get_active() ||
-	      !(chroot->get_session_flags() & chroot::SESSION_CREATE))
+	  /* Create a session using randomly-generated session ID. */
+	  if (ch->get_session_flags() & chroot::SESSION_CREATE)
 	    {
+	      chroot_facet_session_clonable::ptr session_clonable
+		(ch->get_facet<chroot_facet_session_clonable>());
+	      assert(session_clonable);
+
+	      std::string new_session_id;
+
 	      if (!get_session_id().empty())
 		{
-		  session::error e(chroot->get_name(),
-				   session::SET_SESSION_ID,
-				   get_session_id());
-		  log_exception_warning(e);
+		  new_session_id = get_session_id();
 		}
-
-	      set_session_id(chroot->get_name());
-	    }
-	  else
-	    {
-	      if (get_session_id().empty())
+	      else
 		{
-#ifdef HAVE_UUID
-		  uuid_t uuid;
-		  char uuid_str[37];
-		  uuid_generate(uuid);
-		  uuid_unparse(uuid, uuid_str);
-		  uuid_clear(uuid);
-		  std::string session_id(chroot->get_name() + '-' + uuid_str);
-		  set_session_id(session_id);
-#else
-		  std::ostringstream session_id;
-		  session_id.imbue(std::locale::classic());
-		  session_id << chroot->get_name()
-			     << '-' << isodate(time(0))
-			     << '-' << getpid();
-		  set_session_id(session_id.str());
-#endif
+		  new_session_id =
+		    ch->get_name() + '-' + unique_identifier();
 		}
+
+	      // Replace clone of chroot with cloned session.
+
+	      bool in_users = false;
+	      bool in_root_users = false;
+	      bool in_groups = false;
+	      bool in_root_groups = false;
+
+	      get_chroot_membership(chroot,
+				    in_users, in_root_users,
+				    in_groups, in_root_groups);
+
+	      chroot = ch->clone_session(new_session_id,
+					 this->authstat->get_ruser(),
+					 (in_root_users || in_root_groups));
+	      chroot_facet_session::ptr session
+		(chroot->get_facet<chroot_facet_session>());
+	      assert(session);
+	      assert(chroot->get_active());
 	    }
+	  assert(chroot);
 
-	  log_debug(DEBUG_INFO)
-	    << format("Session ID: %1%") % get_session_id() << endl;
-
-	  /* Activate chroot. */
-	  chroot->set_active(true);
-
-	  /* If a chroot mount location has not yet been set, and the
-	     chroot is not a plain chroot, set a mount location with the
-	     session id. */
-	  {
-	    chroot_plain *plain = dynamic_cast<chroot_plain *>(chroot.get());
-	    if (chroot->get_mount_location().empty() &&
-		(plain == 0 || plain->get_run_setup_scripts() == true))
-	      {
-		std::string location(std::string(SCHROOT_MOUNT_DIR) + "/" +
-				     this->session_id);
-		chroot->set_mount_location(location);
-	      }
-	  }
-
-	  log_debug(DEBUG_NOTICE)
-	    << format("Mount Location: %1%") % chroot->get_mount_location()
-	    << endl;
-
-	  /* Chroot types which create a session (e.g. LVM devices)
-	     need the chroot name respecifying. */
-	  if (chroot->get_session_flags() & chroot::SESSION_CREATE)
-	    {
-	      chroot->set_name(this->session_id);
-	      chroot->set_aliases(string_list());
-	    }
-
-	  /* LVM devices need the snapshot device name specifying. */
-	  chroot_lvm_snapshot *snapshot = 0;
-	  if ((snapshot = dynamic_cast<chroot_lvm_snapshot *>(chroot.get())) != 0)
-	    {
-	      std::string dir(dirname(snapshot->get_device(), '/'));
-	      std::string device(dir + "/" + this->session_id);
-	      snapshot->set_snapshot_device(device);
-	    }
+	  // Following authentication success, default child status to
+	  // success so that operations such as beginning, ending and
+	  // recovering sessions will return success unless an
+	  // exception is thrown.
+	  this->child_status = EXIT_SUCCESS;
 
 	  try
 	    {
 	      /* Run setup-start chroot setup scripts. */
 	      setup_chroot(chroot, chroot::SETUP_START);
 	      if (this->session_operation == OPERATION_BEGIN)
-		cout << this->session_id << endl;
+		{
+		  cout << chroot->get_session_id() << endl;
+		}
 
 	      /* Run recover scripts. */
 	      setup_chroot(chroot, chroot::SETUP_RECOVER);
 
-	      try
+	      /* Run session if setup succeeded. */
+	      if (this->session_operation == OPERATION_AUTOMATIC ||
+		  this->session_operation == OPERATION_RUN)
 		{
-		  /* Run exec-start scripts. */
-		  setup_chroot(chroot, chroot::EXEC_START);
-
-		  /* Run session if setup succeeded. */
-		  if (this->session_operation == OPERATION_AUTOMATIC ||
-		      this->session_operation == OPERATION_RUN)
+		  try
 		    {
-		      try
-			{
-			  open_session();
-			  save_termios();
-			  run_chroot(chroot);
-			}
-		      catch (std::runtime_error const& e)
-			{
-			  log_debug(DEBUG_WARNING)
-			    << "Chroot session failed" << endl;
-			  restore_termios();
-			  close_session();
-			  throw;
-			}
-		      restore_termios();
-		      close_session();
+		      this->authstat->open_session();
+		      save_termios();
+		      run_chroot(chroot);
 		    }
-
+		  catch (std::runtime_error const& e)
+		    {
+		      log_debug(DEBUG_WARNING)
+			<< "Chroot session failed" << endl;
+		      restore_termios();
+		      this->authstat->close_session();
+		      throw;
+		    }
+		  restore_termios();
+		  this->authstat->close_session();
 		}
-	      catch (error const& e)
-		{
-		  log_debug(DEBUG_WARNING)
-		    << "Chroot exec scripts or session failed" << endl;
-		  setup_chroot(chroot, chroot::EXEC_STOP);
-		  throw;
-		}
-
-	      /* Run exec-stop scripts whether or not there was an
-		 error. */
-	      setup_chroot(chroot, chroot::EXEC_STOP);
 	    }
 	  catch (error const& e)
 	    {
@@ -655,16 +672,12 @@ session::run_impl ()
 		  log_debug(DEBUG_WARNING)
 		    << "Chroot setup scripts failed during stop" << endl;
 		}
-	      chroot->set_active(false);
 	      throw;
 	    }
 
 	  /* Run setup-stop chroot setup scripts whether or not there
 	     was an error. */
 	  setup_chroot(chroot, chroot::SETUP_STOP);
-
-	  /* Deactivate chroot. */
-	  chroot->set_active(false);
 	}
     }
   catch (error const& e)
@@ -688,7 +701,7 @@ session::get_login_directories () const
 {
   string_list ret;
 
-  std::string const& wd(get_wd());
+  std::string const& wd(this->authstat->get_wd());
   if (!wd.empty())
     {
       // Set specified working directory.
@@ -700,15 +713,15 @@ session::get_login_directories () const
       ret.push_back(this->cwd);
 
       // Set $HOME.
-      environment env = get_pam_environment();
+      environment env = this->authstat->get_auth_environment();
       std::string home;
       if (env.get("HOME", home) &&
 	  std::find(ret.begin(), ret.end(), home) == ret.end())
 	ret.push_back(home);
 
       // Set passwd home.
-      if (std::find(ret.begin(), ret.end(), get_home()) == ret.end())
-	ret.push_back(get_home());
+      if (std::find(ret.begin(), ret.end(), this->authstat->get_home()) == ret.end())
+	ret.push_back(this->authstat->get_home());
 
       // Final fallback to root.
       if (std::find(ret.begin(), ret.end(), "/") == ret.end())
@@ -723,7 +736,7 @@ session::get_command_directories () const
 {
   string_list ret;
 
-  std::string const& wd(get_wd());
+  std::string const& wd(this->authstat->get_wd());
   if (!wd.empty())
     // Set specified working directory.
     ret.push_back(wd);
@@ -737,8 +750,8 @@ session::get_command_directories () const
 std::string
 session::get_shell () const
 {
-  assert (!auth::get_shell().empty());
-  std::string shell = auth::get_shell();
+  assert (!this->authstat->get_shell().empty());
+  std::string shell = this->authstat->get_shell();
 
   try
     {
@@ -783,21 +796,21 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
   std::string shell = get_shell();
   file = shell;
 
-  if (get_environment().empty() &&
+  if (this->authstat->get_environment().empty() &&
       session_chroot->get_command_prefix().empty())
     // Not keeping environment and can setup argv correctly; login shell
     {
-      std::string shellbase = basename(shell, '/');
+      std::string shellbase = basename(shell);
       std::string loginshell = "-" + shellbase;
       command.push_back(loginshell);
 
       log_debug(DEBUG_NOTICE)
 	<< format("Running login shell: %1%") % shell << endl;
-      if (get_uid() == 0 || get_ruid() != get_uid())
+      if (this->authstat->get_uid() == 0 || this->authstat->get_ruid() != this->authstat->get_uid())
 	syslog(LOG_USER|LOG_NOTICE,
 	       "[%s chroot] (%s->%s) Running login shell: '%s'",
 	       session_chroot->get_name().c_str(),
-	       get_ruser().c_str(), get_user().c_str(),
+	       this->authstat->get_ruser().c_str(), this->authstat->get_user().c_str(),
 	       shell.c_str());
     }
   else
@@ -805,20 +818,20 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
       command.push_back(shell);
       log_debug(DEBUG_NOTICE)
 	<< format("Running shell: %1%") % shell << endl;
-      if (get_uid() == 0 || get_ruid() != get_uid())
+      if (this->authstat->get_uid() == 0 || this->authstat->get_ruid() != this->authstat->get_uid())
 	syslog(LOG_USER|LOG_NOTICE,
 	       "[%s chroot] (%s->%s) Running shell: '%s'",
 	       session_chroot->get_name().c_str(),
-	       get_ruser().c_str(), get_user().c_str(),
+	       this->authstat->get_ruser().c_str(), this->authstat->get_user().c_str(),
 	       shell.c_str());
     }
 
-  if (get_verbosity() != auth::VERBOSITY_QUIET)
+  if (this->authstat->get_verbosity() == auth::VERBOSITY_VERBOSE)
     {
       std::string format_string;
-      if (get_ruid() == get_uid())
+      if (this->authstat->get_ruid() == this->authstat->get_uid())
 	{
-	  if (get_environment().empty() &&
+	  if (this->authstat->get_environment().empty() &&
 	      session_chroot->get_command_prefix().empty())
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %4% = command
@@ -830,7 +843,7 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 	}
       else
 	{
-	  if (get_environment().empty() &&
+	  if (this->authstat->get_environment().empty() &&
 	      session_chroot->get_command_prefix().empty())
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %2% = user name
@@ -849,7 +862,7 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 
       format fmt(format_string);
       fmt % session_chroot->get_name()
-	% get_ruser() % get_user()
+	% this->authstat->get_ruser() % this->authstat->get_user()
 	% shell;
       log_info() << fmt << endl;
     }
@@ -861,7 +874,7 @@ session::get_user_command (sbuild::chroot::ptr& session_chroot,
 			   string_list&         command) const
 {
   /* Search for program in path. */
-  environment env = get_pam_environment();
+  environment env = this->authstat->get_auth_environment();
   std::string path;
   if (!env.get("PATH", path))
     path.clear();
@@ -872,14 +885,14 @@ session::get_user_command (sbuild::chroot::ptr& session_chroot,
   std::string commandstring = string_list_to_string(command, " ");
   log_debug(DEBUG_NOTICE)
     << format("Running command: %1%") % commandstring << endl;
-  if (get_uid() == 0 || get_ruid() != get_uid())
+  if (this->authstat->get_uid() == 0 || this->authstat->get_ruid() != this->authstat->get_uid())
     syslog(LOG_USER|LOG_NOTICE, "[%s chroot] (%s->%s) Running command: \"%s\"",
-	   session_chroot->get_name().c_str(), get_ruser().c_str(), get_user().c_str(), commandstring.c_str());
+	   session_chroot->get_name().c_str(), this->authstat->get_ruser().c_str(), this->authstat->get_user().c_str(), commandstring.c_str());
 
-  if (get_verbosity() != auth::VERBOSITY_QUIET)
+  if (this->authstat->get_verbosity() == auth::VERBOSITY_VERBOSE)
     {
       std::string format_string;
-      if (get_ruid() == get_uid())
+      if (this->authstat->get_ruid() == this->authstat->get_uid())
 	// TRANSLATORS: %1% = chroot name
 	// TRANSLATORS: %4% = command
 	format_string = _("[%1% chroot] Running command: \"%4%\"");
@@ -893,7 +906,7 @@ session::get_user_command (sbuild::chroot::ptr& session_chroot,
 
       format fmt(format_string);
       fmt % session_chroot->get_name()
-	% get_ruser() % get_user()
+	% this->authstat->get_ruser() % this->authstat->get_user()
 	% commandstring;
       log_info() << fmt << endl;
     }
@@ -915,27 +928,19 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
 	 setup_type == chroot::SETUP_RECOVER) ||
 	(this->session_operation == OPERATION_END &&
 	 setup_type == chroot::SETUP_STOP) ||
-	(this->session_operation == OPERATION_RUN &&
-	 (setup_type == chroot::EXEC_START ||
-	  setup_type == chroot::EXEC_STOP)) ||
 	(this->session_operation == OPERATION_AUTOMATIC &&
 	 (setup_type == chroot::SETUP_START ||
-	  setup_type == chroot::SETUP_STOP  ||
-	  setup_type == chroot::EXEC_START   ||
-	  setup_type == chroot::EXEC_STOP))))
+	  setup_type == chroot::SETUP_STOP))))
     return;
 
   // Don't clean up chroot on a lock failure--it's actually in use.
   if (this->lock_status == false)
     return;
 
-  if (((setup_type == chroot::SETUP_START   ||
-	setup_type == chroot::SETUP_RECOVER ||
-	setup_type == chroot::SETUP_STOP) &&
-       session_chroot->get_run_setup_scripts() == false) ||
-      ((setup_type == chroot::EXEC_START ||
-	setup_type == chroot::EXEC_STOP) &&
-       session_chroot->get_run_exec_scripts() == false))
+  if ((setup_type == chroot::SETUP_START   ||
+       setup_type == chroot::SETUP_RECOVER ||
+       setup_type == chroot::SETUP_STOP) &&
+      session_chroot->get_run_setup_scripts() == false)
     return;
 
   if (setup_type == chroot::SETUP_START)
@@ -967,10 +972,6 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
     setup_type_string = "setup-recover";
   else if (setup_type == chroot::SETUP_STOP)
     setup_type_string = "setup-stop";
-  else if (setup_type == chroot::EXEC_START)
-    setup_type_string = "exec-start";
-  else if (setup_type == chroot::EXEC_STOP)
-    setup_type_string = "exec-stop";
 
   std::string chroot_status_string;
   if (this->chroot_status)
@@ -987,10 +988,18 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
      chroot type. */
   environment env;
   session_chroot->setup_env(env);
-  env.add("AUTH_USER", get_user());
+  env.add("AUTH_USER", this->authstat->get_user());
+  env.add("AUTH_RUSER", this->authstat->get_ruser());
+  env.add("AUTH_RGROUP", this->authstat->get_rgroup());
+  env.add("AUTH_UID", this->authstat->get_uid());
+  env.add("AUTH_GID", this->authstat->get_gid());
+  env.add("AUTH_RUID", this->authstat->get_ruid());
+  env.add("AUTH_RGID", this->authstat->get_rgid());
+  env.add("AUTH_HOME", this->authstat->get_home());
+  env.add("AUTH_SHELL", this->authstat->get_shell());
   {
     const char *verbosity = 0;
-    switch (get_verbosity())
+    switch (this->authstat->get_verbosity())
       {
       case auth::VERBOSITY_QUIET:
 	verbosity = "quiet";
@@ -1003,7 +1012,7 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
 	break;
       default:
 	log_debug(DEBUG_CRITICAL) << format("Invalid verbosity level: %1%, falling back to 'normal'")
-	  % static_cast<int>(get_verbosity())
+	  % static_cast<int>(this->authstat->get_verbosity())
 		     << endl;
 	verbosity = "normal";
 	break;
@@ -1014,17 +1023,12 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
   env.add("MOUNT_DIR", SCHROOT_MOUNT_DIR);
   env.add("LIBEXEC_DIR", SCHROOT_LIBEXEC_DIR);
   env.add("PID", getpid());
-  env.add("SESSION_ID", this->session_id);
+  env.add("SESSION_ID", session_chroot->get_session_id());
 
-  run_parts rp((setup_type == chroot::SETUP_START ||
-		setup_type == chroot::SETUP_RECOVER ||
-		setup_type == chroot::SETUP_STOP)
-	       ? SCHROOT_CONF_SETUP_D // Setup directory
-	       : SCHROOT_CONF_EXEC_D, // Execution directory
+  run_parts rp(SCHROOT_CONF_SETUP_D,
 	       true, true, 022);
-  rp.set_reverse((setup_type == chroot::SETUP_STOP ||
-		  setup_type == chroot::EXEC_STOP));
-  rp.set_verbose(get_verbosity() == auth::VERBOSITY_VERBOSE);
+  rp.set_reverse(setup_type == chroot::SETUP_STOP);
+  rp.set_verbose(this->authstat->get_verbosity() == auth::VERBOSITY_VERBOSE);
 
   log_debug(DEBUG_INFO) << rp << std::endl;
 
@@ -1097,30 +1101,39 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
 {
   assert(!session_chroot->get_name().empty());
 
-  assert(!get_user().empty());
+  assert(!this->authstat->get_user().empty());
   assert(!get_shell().empty());
-  assert(auth::pam != 0); // PAM must be initialised
+  assert(this->authstat->is_initialised()); // PAM must be initialised
 
   // Store before chroot call.
-  this->cwd = getcwd();
+  this->cwd = sbuild::getcwd();
   log_debug(DEBUG_INFO) << "CWD=" << this->cwd << std::endl;
 
   std::string location(session_chroot->get_path());
   log_debug(DEBUG_INFO) << "location=" << location << std::endl;
 
   /* Set group ID and supplementary groups */
-  if (setgid (get_gid()))
-    throw error(get_gid(), GROUP_SET, strerror(errno));
-  log_debug(DEBUG_NOTICE) << "Set GID=" << get_gid() << std::endl;
-  if (initgroups (get_user().c_str(), get_gid()))
+  if (setgid (this->authstat->get_gid()))
+    throw error(this->authstat->get_gid(), GROUP_SET, strerror(errno));
+  log_debug(DEBUG_NOTICE) << "Set GID=" << this->authstat->get_gid() << std::endl;
+  if (initgroups (this->authstat->get_user().c_str(), this->authstat->get_gid()))
     throw error(GROUP_SET_SUP, strerror(errno));
   log_debug(DEBUG_NOTICE) << "Set supplementary groups" << std::endl;
 
   /* Set the process execution domain. */
   /* Will throw on failure. */
-  session_chroot->get_persona().set();
-  log_debug(DEBUG_NOTICE) << "Set personality="
-			  << session_chroot->get_persona()<< std::endl;
+  chroot_facet_personality::const_ptr pfac =
+    session_chroot->get_facet<chroot_facet_personality>();
+  if (pfac)
+    {
+      pfac->get_persona().set();
+      log_debug(DEBUG_NOTICE) << "Set personality="
+			      << pfac->get_persona() << std::endl;
+    }
+  else
+    {
+      log_debug(DEBUG_NOTICE) << "Personality support unavailable" << std::endl;
+    }
 
   /* Enter the chroot */
   if (chdir (location.c_str()))
@@ -1131,16 +1144,16 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
   log_debug(DEBUG_NOTICE) << "Changed root to " << location << std::endl;
 
   /* Set uid and check we are not still root */
-  if (setuid (get_uid()))
-    throw error(get_uid(), USER_SET, strerror(errno));
-  log_debug(DEBUG_NOTICE) << "Set UID=" << get_uid() << std::endl;
-  if (!setuid (0) && get_uid())
+  if (setuid (this->authstat->get_uid()))
+    throw error(this->authstat->get_uid(), USER_SET, strerror(errno));
+  log_debug(DEBUG_NOTICE) << "Set UID=" << this->authstat->get_uid() << std::endl;
+  if (!setuid (0) && this->authstat->get_uid())
     throw error(ROOT_DROP);
-  if (get_uid())
+  if (this->authstat->get_uid())
     log_debug(DEBUG_NOTICE) << "Dropped root privileges" << std::endl;
 
   std::string file;
-  string_list command(auth::get_command());
+  string_list command(this->authstat->get_command());
 
   string_list dlist;
   if (command.empty() ||
@@ -1187,17 +1200,18 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
   /* Set up environment */
   environment env;
   env.set_filter(session_chroot->get_environment_filter());
-  env += get_pam_environment();
+  env += this->authstat->get_auth_environment();
 
   // Add equivalents to sudo's SUDO_USER, SUDO_UID, SUDO_GID, and
   // SUDO_COMMAND.
   env.add(std::make_pair("SCHROOT_COMMAND",
 			 string_list_to_string(command, " ")));
-  env.add(std::make_pair("SCHROOT_USER", get_ruser()));
-  env.add(std::make_pair("SCHROOT_GROUP", get_rgroup()));
-  env.add("SCHROOT_UID", get_ruid());
-  env.add("SCHROOT_GID", get_rgid());
-
+  env.add(std::make_pair("SCHROOT_USER", this->authstat->get_ruser()));
+  env.add(std::make_pair("SCHROOT_GROUP", this->authstat->get_rgroup()));
+  env.add("SCHROOT_UID", this->authstat->get_ruid());
+  env.add("SCHROOT_GID", this->authstat->get_rgid());
+  // Add session ID.
+  env.add("SCHROOT_SESSION_ID", session_chroot->get_session_id());
 
   log_debug(DEBUG_INFO) << "Set environment:\n" << env;
 
@@ -1207,7 +1221,12 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
   // Add command prefix.
   string_list full_command(session_chroot->get_command_prefix());
   if (full_command.size() > 0)
-    file = full_command[0];
+    {
+      std::string path;
+      if (!env.get("PATH", path))
+	path.clear();
+      file = find_program_in_path(full_command[0], path, "");
+    }
   for (string_list::const_iterator pos = command.begin();
        pos != command.end();
        ++pos)
